/*
 * Created on Jan 29, 2005
 */
package com.aggressivesoftware.general;

import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.ComponentColorModel;
import java.awt.image.DirectColorModel;
import java.awt.image.ImageProducer;
import java.awt.image.IndexColorModel;
import java.awt.image.WritableRaster;

import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;

import com.sun.jimi.core.JimiException;
import com.sun.jimi.core.util.ColorReducer;

/**
 * @author Michael Wood
 */
public class ConvertSWT 
{
	/**
	 * 
	 */
	public static ImageData convertToSWT(BufferedImage bufferedImage) 
	{
		if (bufferedImage.getColorModel() instanceof DirectColorModel) 
		{
			// Copyright (c) 2000, 2004 IBM Corporation and others.
			DirectColorModel colorModel = (DirectColorModel)bufferedImage.getColorModel();
			
			PaletteData palette = new PaletteData(colorModel.getRedMask(), colorModel.getGreenMask(), colorModel.getBlueMask());
			ImageData data = new ImageData(bufferedImage.getWidth(), bufferedImage.getHeight(), colorModel.getPixelSize(), palette);
			WritableRaster raster = bufferedImage.getRaster();
			
			int[] pixelArray = new int[3];
			for (int y = 0; y < data.height; y++) {
				for (int x = 0; x < data.width; x++) {
					raster.getPixel(x, y, pixelArray);
					int pixel = palette.getPixel(new RGB(pixelArray[0], pixelArray[1], pixelArray[2]));
					data.setPixel(x, y, pixel);
				}
			}		
			return data;		
		} 
		else if (bufferedImage.getColorModel() instanceof IndexColorModel) 
		{
			// Copyright (c) 2000, 2004 IBM Corporation and others.
			IndexColorModel colorModel = (IndexColorModel)bufferedImage.getColorModel();

			int size = colorModel.getMapSize();
			byte[] reds = new byte[size];
			byte[] greens = new byte[size];
			byte[] blues = new byte[size];
			colorModel.getReds(reds);
			colorModel.getGreens(greens);
			colorModel.getBlues(blues);
			RGB[] rgbs = new RGB[size];
			for (int i = 0; i < rgbs.length; i++) {
				rgbs[i] = new RGB(reds[i] & 0xFF, greens[i] & 0xFF, blues[i] & 0xFF);
			}
			
			PaletteData palette = new PaletteData(rgbs);
			ImageData data = new ImageData(bufferedImage.getWidth(), bufferedImage.getHeight(), colorModel.getPixelSize(), palette);
			data.transparentPixel = colorModel.getTransparentPixel();
			WritableRaster raster = bufferedImage.getRaster();
			
			int[] pixelArray = new int[1];
			for (int y = 0; y < data.height; y++) {
				for (int x = 0; x < data.width; x++) {
					raster.getPixel(x, y, pixelArray);
					data.setPixel(x, y, pixelArray[0]);
				}
			}
			return data;
		}
		else if (bufferedImage.getColorModel() instanceof ComponentColorModel) 
		{
			// Written by Michael Wood
			// This is a hideous hack.  Sorry.
			// ColorModel: #pixelBits    = 8 
			//             numComponents = 1 
			//             color space   = java.awt.color.ICC_ColorSpace@73a34b
			//             transparency  = 1 
			//             has alpha     = false 
			//             isAlphaPre    = false
			ComponentColorModel color_model = (ComponentColorModel)bufferedImage.getColorModel();
						
			RGB[] rgbs = new RGB[2];
			rgbs[0] = new RGB(0x00, 0x00, 0x00);
			rgbs[1] = new RGB(0xEC, 0xE9, 0xD8);
			PaletteData palette = new PaletteData(rgbs);
			ImageData data = new ImageData(bufferedImage.getWidth(), bufferedImage.getHeight(), color_model.getPixelSize(), palette);
			WritableRaster raster = bufferedImage.getRaster();
			
			int[] pixelArray = new int[1];
			int pixel = 0;
			for (int y = 0; y < data.height; y++) {
				for (int x = 0; x < data.width; x++) {
					raster.getPixel(x, y, pixelArray);
					if (pixelArray[0] > 0) { pixel = 1; }
					else { pixel = 0; }
					data.setPixel(x, y, pixel);
				}
			}	
			return data;	
		}
		System.out.println("failed to convert to swt");
		return null;
	}
	
	// Copyright (c) 2000, 2004 IBM Corporation and others.
	public static BufferedImage convertToAWT(ImageData data) {
		ColorModel colorModel = null;
		PaletteData palette = data.palette;
		if (palette.isDirect) {
			colorModel = new DirectColorModel(data.depth, palette.redMask, palette.greenMask, palette.blueMask);
			BufferedImage bufferedImage = new BufferedImage(colorModel, colorModel.createCompatibleWritableRaster(data.width, data.height), false, null);
			WritableRaster raster = bufferedImage.getRaster();
			int[] pixelArray = new int[3];
			for (int y = 0; y < data.height; y++) {
				for (int x = 0; x < data.width; x++) {
					int pixel = data.getPixel(x, y);
					RGB rgb = palette.getRGB(pixel);
					pixelArray[0] = rgb.red;
					pixelArray[1] = rgb.green;
					pixelArray[2] = rgb.blue;
					raster.setPixels(x, y, 1, 1, pixelArray);
				}
			}
			return bufferedImage;
		} else {
			RGB[] rgbs = palette.getRGBs();
			byte[] red = new byte[rgbs.length];
			byte[] green = new byte[rgbs.length];
			byte[] blue = new byte[rgbs.length];
			for (int i = 0; i < rgbs.length; i++) {
				RGB rgb = rgbs[i];
				red[i] = (byte)rgb.red;
				green[i] = (byte)rgb.green;
				blue[i] = (byte)rgb.blue;
			}
			if (data.transparentPixel != -1) {
				colorModel = new IndexColorModel(data.depth, rgbs.length, red, green, blue, data.transparentPixel);
			} else {
				colorModel = new IndexColorModel(data.depth, rgbs.length, red, green, blue);
			}		
			BufferedImage bufferedImage = new BufferedImage(colorModel, colorModel.createCompatibleWritableRaster(data.width, data.height), false, null);
			WritableRaster raster = bufferedImage.getRaster();
			int[] pixelArray = new int[1];
			for (int y = 0; y < data.height; y++) {
				for (int x = 0; x < data.width; x++) {
					int pixel = data.getPixel(x, y);
					pixelArray[0] = pixel;
					raster.setPixel(x, y, pixelArray);
				}
			}
			return bufferedImage;
		}
	}
	
	public static ImageProducer getReducedFromSWT(ImageData image_data)
	{
		// reduce colors using JIMI
		ColorReducer reducer = new ColorReducer(256, true);
		try { return reducer.getColorReducedImageProducer((java.awt.Image)ConvertSWT.convertToAWT(image_data)); }
		catch (JimiException e) { e.printStackTrace(); }
		return null;
	}
}